/****************************************************************************
    UART.c
    ------
    bungsprojekt zum Empfang von Daten mit dem UART.
    Eine LED an Pin 0 von Port B soll gesteuert werden beim Empfang folgender
    Zeichen:
        '1' - LED ein.
        '0' - LED aus.


    Autor:    Christian Schifferle
    Datum:    November 2002
    Software: AVR-GCC
    needed
****************************************************************************/

#include <io.h>	// Wir brauchen Zugriff auf die I/O's des Controllers

#define F_CPU            8000000      /* 8Mhz */
#define UART_BAUD_RATE      9600      /* 9600 baud */

// Berechnung des Inhalts des UBBR-Registers
#define UART_BAUD_SELECT (F_CPU/(UART_BAUD_RATE*16l)-1)

// Datentypdefinitionen
typedef unsigned char	BYTE;
typedef unsigned short	WORD;



int main (void)
/****************************************************************************
    Hier ist der Programmeinstiegspunkt.
*/
{
    /* UART Control Register laden. Wir verwenden (noch)
       keine Interrupts. Hier wollen wir nur empfangen, also wird
       nur das Receiver Enable Flag gesetzt. */
    outp ((1 << RXEN), UCR);

    /* Baudrate einstellen */
    outp ((BYTE) UART_BAUD_SELECT, UBRR);          


    // Port B als Ausgang konfigurieren
    outp (0xff, DDRB);

    // Hier machen wir einmal etwas anderes zur Realisierung der
    // Endlosschleife. Wir setzen zu Beginn der Schleife ein Label.
    // Am Schleifenende springen wir direkt zu diesem Label.
    // Nicht schn, aber unheimlich schnell.
LOOP:
    // Prfen, ob ein Zeichen empfangen wurde.

    if (inp (USR) & (1 << RXC)) {     // Bit RXC im USR gesetzt ?
        switch (inp (UDR)) {          // Datenregister auslesen
            case '1': // LED einschalten
                sbi (PORTB, PINB0);
                break;
            case '0': // LED ausschalten
                cbi (PORTB, PINB0);
                break;
            default:  // Bei allen anderen Zeichen nichts tun
                break;
    }
}
    
goto LOOP;

} /* end of main */
